@@ -1,4 +1,6 @@
+# permissions imports
 import permissions.utils
+from permissions.exceptions import Unauthorized
 
 class PermissionBase(object):
     """Mix-in class for permissions.
@@ -55,7 +57,7 @@ class PermissionBase(object):
         return permissions.utils.has_permission(self, user, permission, roles)
 
     def check_permission(self, user, permission, roles=None):
-        """Raise Unauthorized if the the passed user hasn't passed permission 
+        """Raise Unauthorized if the the passed user hasn't passed permission
         for this instance.
 
         **Parameters:**
@@ -75,7 +77,7 @@ class PermissionBase(object):
             roles = []
 
         if not self.has_permission(user, permission, roles):
-            raise Unauthorized("User %s doesn't have permission %s for object %s" % (user, permission, obj.slug))
+            raise Unauthorized("User '%s' doesn't have permission '%s' for object '/%s' (%s)." % (user, permission, self.slug, self.__class__.name))
 
     def add_inheritance_block(self, permission):
         """Adds an inheritance block for the passed permission.
@@ -124,7 +126,7 @@ class PermissionBase(object):
         return permissions.utils.add_local_role(self, principal, role)
 
     def get_roles(self, principal):
-        """Returns local roles for passed principal (user or group).
+        """Returns *direct* local roles for passed principal (user or group).
         """
         return permissions.utils.get_local_roles(self, principal)
 
@@ -150,4 +152,4 @@ class PermissionBase(object):
             The principal (user or group) from which all local roles are
             removed.
         """
-        return permissions.utils.remove_local_roles(self, principal)
\ No newline at end of file
+        return permissions.utils.remove_local_roles(self, principal)
