@@ -2,6 +2,7 @@
 from django.contrib.flatpages.models import FlatPage
 from django.contrib.auth.models import Group
 from django.contrib.auth.models import User
+from django.conf import settings
 from django.core.urlresolvers import reverse
 from django.test import TestCase
 from django.test.client import Client
@@ -14,6 +15,37 @@ from permissions.models import Role
 
 import permissions.utils
 
+class BackendTestCase(TestCase):
+    """
+    """
+    def setUp(self):
+        """
+        """
+        settings.AUTHENTICATION_BACKENDS = (
+            'django.contrib.auth.backends.ModelBackend',
+            'permissions.backend.ObjectPermissionsBackend',
+        )
+
+        self.role_1 = permissions.utils.register_role("Role 1")
+        self.user = User.objects.create(username="john")
+        self.page_1 = FlatPage.objects.create(url="/page-1/", title="Page 1")
+        self.view = permissions.utils.register_permission("View", "view")
+
+        # Add user to role
+        self.role_1.add_principal(self.user)
+
+    def test_has_perm(self):
+        """Tests has perm of the backend.
+        """
+        result = self.user.has_perm(self.view, self.page_1)
+        self.assertEqual(result, False)
+
+        # assign view permission to role 1
+        permissions.utils.grant_permission(self.page_1, self.role_1, self.view)
+
+        result = self.user.has_perm("view", self.page_1)
+        self.assertEqual(result, True)
+
 class RoleTestCase(TestCase):
     """
     """
@@ -34,24 +66,44 @@ class RoleTestCase(TestCase):
     def test_getter(self):
         """
         """
+        # Group
         result = permissions.utils.get_group(self.group.id)
         self.assertEqual(result, self.group)
 
         result = permissions.utils.get_group(42)
         self.assertEqual(result, None)
 
+        result = permissions.utils.get_group(self.group.name)
+        self.assertEqual(result, self.group)
+
+        result = permissions.utils.get_group("Not Existing")
+        self.assertEqual(result, None)
+
+        # Role
         result = permissions.utils.get_role(self.role_1.id)
         self.assertEqual(result, self.role_1)
 
         result = permissions.utils.get_role(42)
         self.assertEqual(result, None)
 
+        result = permissions.utils.get_role(self.role_1.name)
+        self.assertEqual(result, self.role_1)
+
+        result = permissions.utils.get_role("Not Existing")
+        self.assertEqual(result, None)
+
         result = permissions.utils.get_user(self.user.id)
         self.assertEqual(result, self.user)
 
         result = permissions.utils.get_user(42)
         self.assertEqual(result, None)
 
+        result = permissions.utils.get_user(self.user.username)
+        self.assertEqual(result, self.user)
+
+        result = permissions.utils.get_user("Not Existing")
+        self.assertEqual(result, None)
+
     def test_global_roles_user(self):
         """
         """
@@ -64,14 +116,15 @@ class RoleTestCase(TestCase):
         self.assertEqual(result, False)
 
         result = permissions.utils.get_roles(self.user)
-        self.assertEqual(result, [self.role_1])
+        self.assertEqual(list(result), [self.role_1])
 
         # Add role 2
         result = permissions.utils.add_role(self.user, self.role_2)
         self.assertEqual(result, True)
 
+        delattr(self.user, "roles")
         result = permissions.utils.get_roles(self.user)
-        self.assertEqual(result, [self.role_1, self.role_2])
+        self.assertEqual(list(result), [self.role_1, self.role_2])
 
         # Remove role 1
         result = permissions.utils.remove_role(self.user, self.role_1)
@@ -81,15 +134,17 @@ class RoleTestCase(TestCase):
         result = permissions.utils.remove_role(self.user, self.role_1)
         self.assertEqual(result, False)
 
+        delattr(self.user, "roles")
         result = permissions.utils.get_roles(self.user)
-        self.assertEqual(result, [self.role_2])
+        self.assertEqual(list(result), [self.role_2])
 
         # Remove role 2
         result = permissions.utils.remove_role(self.user, self.role_2)
         self.assertEqual(result, True)
 
+        delattr(self.user, "roles")
         result = permissions.utils.get_roles(self.user)
-        self.assertEqual(result, [])
+        self.assertEqual(list(result), [])
 
     def test_global_roles_group(self):
         """
@@ -102,14 +157,14 @@ class RoleTestCase(TestCase):
         result = permissions.utils.add_role(self.group, self.role_1)
         self.assertEqual(result, False)
 
-        result = permissions.utils.get_roles(self.group)
+        result = permissions.utils.get_global_roles(self.group)
         self.assertEqual(result, [self.role_1])
 
         # Add role 2
         result = permissions.utils.add_role(self.group, self.role_2)
         self.assertEqual(result, True)
 
-        result = permissions.utils.get_roles(self.group)
+        result = permissions.utils.get_global_roles(self.group)
         self.assertEqual(result, [self.role_1, self.role_2])
 
         # Remove role 1
@@ -120,14 +175,14 @@ class RoleTestCase(TestCase):
         result = permissions.utils.remove_role(self.group, self.role_1)
         self.assertEqual(result, False)
 
-        result = permissions.utils.get_roles(self.group)
+        result = permissions.utils.get_global_roles(self.group)
         self.assertEqual(result, [self.role_2])
 
         # Remove role 2
         result = permissions.utils.remove_role(self.group, self.role_2)
         self.assertEqual(result, True)
 
-        result = permissions.utils.get_roles(self.group)
+        result = permissions.utils.get_global_roles(self.group)
         self.assertEqual(result, [])
 
     def test_remove_roles_user(self):
@@ -142,14 +197,15 @@ class RoleTestCase(TestCase):
         self.assertEqual(result, True)
 
         result = permissions.utils.get_roles(self.user)
-        self.assertEqual(result, [self.role_1, self.role_2])
+        self.assertEqual(list(result), [self.role_1, self.role_2])
 
         # Remove roles
         result = permissions.utils.remove_roles(self.user)
         self.assertEqual(result, True)
 
+        delattr(self.user, "roles")
         result = permissions.utils.get_roles(self.user)
-        self.assertEqual(result, [])
+        self.assertEqual(list(result), [])
 
         # Remove roles
         result = permissions.utils.remove_roles(self.user)
@@ -166,14 +222,14 @@ class RoleTestCase(TestCase):
         result = permissions.utils.add_role(self.group, self.role_2)
         self.assertEqual(result, True)
 
-        result = permissions.utils.get_roles(self.group)
+        result = permissions.utils.get_global_roles(self.group)
         self.assertEqual(result, [self.role_1, self.role_2])
 
         # Remove roles
         result = permissions.utils.remove_roles(self.group)
         self.assertEqual(result, True)
 
-        result = permissions.utils.get_roles(self.group)
+        result = permissions.utils.get_global_roles(self.group)
         self.assertEqual(result, [])
 
         # Remove roles
@@ -432,14 +488,14 @@ class RoleTestCase(TestCase):
         self.assertEqual(result[0].username, "john")
         self.assertEqual(result[1].username, "jane")
         self.assertEqual(len(result), 2)
-        
+
     def test_get_users_3(self):
         """
         """
         self.role_1.add_principal(self.user)
         result = self.role_1.get_users()
         self.assertEqual(result, [self.user])
-        
+
         # Add same user again
         self.role_1.add_principal(self.user)
         result = self.role_1.get_users()
@@ -459,7 +515,7 @@ class PermissionTestCase(TestCase):
         self.user.save()
 
         self.page_1 = FlatPage.objects.create(url="/page-1/", title="Page 1")
-        self.page_2 = FlatPage.objects.create(url="/page-1/", title="Page 2")
+        self.page_2 = FlatPage.objects.create(url="/page-2/", title="Page 2")
 
         self.permission = permissions.utils.register_permission("View", "view")
 
@@ -533,6 +589,8 @@ class PermissionTestCase(TestCase):
 
         permissions.utils.grant_permission(self.page_1, self.role_2, self.permission)
         permissions.utils.add_local_role(self.page_1, self.user, self.role_2)
+        # Remove cache. This is done automatically by Django for every new request
+        delattr(self.user, "roles")
 
         result = permissions.utils.has_permission(self.page_1, self.user, "view")
         self.assertEqual(result, True)
@@ -760,4 +818,4 @@ def create_request():
     user.save()
     request.user = user
 
-    return request
\ No newline at end of file
+    return request
