@@ -1,5 +1,9 @@
+# python imports
+import warnings
+
 # django imports
 from django.db import IntegrityError
+from django.db import connection
 from django.db.models import Q
 from django.contrib.auth.models import User
 from django.contrib.auth.models import Group
@@ -7,6 +11,7 @@ from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist
 
 # permissions imports
+from permissions.exceptions import Unauthorized
 from permissions.models import ObjectPermission
 from permissions.models import ObjectPermissionInheritanceBlock
 from permissions.models import Permission
@@ -28,13 +33,13 @@ def add_role(principal, role):
     """
     if isinstance(principal, User):
         try:
-            ppr = PrincipalRoleRelation.objects.get(user=principal, role=role, content_id=None, content_type=None)
+            PrincipalRoleRelation.objects.get(user=principal, role=role, content_id=None, content_type=None)
         except PrincipalRoleRelation.DoesNotExist:
             PrincipalRoleRelation.objects.create(user=principal, role=role)
             return True
     else:
         try:
-            ppr = PrincipalRoleRelation.objects.get(group=principal, role=role, content_id=None, content_type=None)
+            PrincipalRoleRelation.objects.get(group=principal, role=role, content_id=None, content_type=None)
         except PrincipalRoleRelation.DoesNotExist:
             PrincipalRoleRelation.objects.create(group=principal, role=role)
             return True
@@ -58,13 +63,13 @@ def add_local_role(obj, principal, role):
     ctype = ContentType.objects.get_for_model(obj)
     if isinstance(principal, User):
         try:
-            ppr = PrincipalRoleRelation.objects.get(user=principal, role=role, content_id=obj.id, content_type=ctype)
+            PrincipalRoleRelation.objects.get(user=principal, role=role, content_id=obj.id, content_type=ctype)
         except PrincipalRoleRelation.DoesNotExist:
             PrincipalRoleRelation.objects.create(user=principal, role=role, content=obj)
             return True
     else:
         try:
-            ppr = PrincipalRoleRelation.objects.get(group=principal, role=role, content_id=obj.id, content_type=ctype)
+            PrincipalRoleRelation.objects.get(group=principal, role=role, content_id=obj.id, content_type=ctype)
         except PrincipalRoleRelation.DoesNotExist:
             PrincipalRoleRelation.objects.create(group=principal, role=role, content=obj)
             return True
@@ -98,7 +103,7 @@ def remove_role(principal, role):
     return True
 
 def remove_local_role(obj, principal, role):
-    """Removes role from obj and principle.
+    """Removes role from passed object and principle.
 
     **Parameters:**
 
@@ -176,44 +181,93 @@ def remove_local_roles(obj, principal):
     else:
         return False
 
-def get_roles(principal, obj=None):
-    """Returns all roles of passed user for passed content object. This takes
-    direct and roles via a group into account. If an object is passed local
-    roles will also added.
+
+def get_roles(user, obj=None):
+    """Returns *all* roles of the passed user.
+
+    This takes direct roles and roles via the user's groups into account.
+
+    If an object is passed local roles will also added. Then all local roles
+    from all ancestors and all user's groups are also taken into account.
+
+    This is the method to use if one want to know whether the passed user
+    has a role in general (for the passed object).
 
     **Parameters:**
 
+    user
+        The user for which the roles are returned.
+
     obj
-        The object from which the roles are removed.
+        The object for which local roles will returned.
 
-    principal
-        The principal (user or group) from which the roles are removed.
     """
-    roles = get_global_roles(principal)
+    # Cached roles
+    obj_id = str(obj.id) if obj else "0"
+    try:
+        return user.roles[obj_id]
+    except (AttributeError, KeyError):
+        pass
 
-    if obj is not None:
-        roles.extend(get_local_roles(obj, principal))
+    groups = user.groups.all()
+    groups_ids_str = ", ".join([str(g.id) for g in groups])
 
-    if isinstance(principal, User):
-        for group in principal.groups.all():
-            if obj is not None:
-                roles.extend(get_local_roles(obj, group))
-            roles.extend(get_roles(group))
+    if groups_ids_str:
+        prrs = PrincipalRoleRelation.objects.filter(
+            Q(user_id=user.id) | Q(group_id__in=groups_ids_str), content_id=None
+        ).values("role_id")
+    else:
+        prrs = PrincipalRoleRelation.objects.filter(user_id=user.id, content_id=None).values("role_id")
+
+    role_ids = [ppr["role_id"] for ppr in prrs]
+
+    # Local roles for user and the user's groups and all ancestors of the
+    # passed object.
+    while obj:
+        ctype = ContentType.objects.get_for_model(obj)
+
+        if groups_ids_str:
+            prrs = PrincipalRoleRelation.objects.filter(
+                Q(user_id=user.id) | Q(group_id__in=groups_ids_str), content_id=obj.id, content_type_id=ctype.id
+            ).values("role_id")
+        else:
+            prrs = PrincipalRoleRelation.objects.filter(
+                user_id=user.id, content_id=obj.id, content_type_id=ctype.id
+            ).values("role_id")
+
+        for prr in prrs:
+            role_ids.append(prr["role_id"])
+
+        try:
+            obj = obj.get_parent_for_permissions()
+        except AttributeError:
+            obj = None
+
+    roles = Role.objects.filter(pk__in=role_ids)
+
+    # Cache roles per object
+    if not getattr(user, "roles", False):
+        user.roles = {}
+    user.roles[obj_id] = roles
 
     return roles
 
+
 def get_global_roles(principal):
-    """Returns global roles of passed principal (user or group).
+    """Returns *direct* global roles of passed principal (user or group).
     """
     if isinstance(principal, User):
         return [prr.role for prr in PrincipalRoleRelation.objects.filter(
             user=principal, content_id=None, content_type=None)]
     else:
+        if isinstance(principal, Group):
+            principal = (principal,)
         return [prr.role for prr in PrincipalRoleRelation.objects.filter(
-            group=principal, content_id=None, content_type=None)]
+            group__in=principal, content_id=None, content_type=None)]
+
 
 def get_local_roles(obj, principal):
-    """Returns local for passed user and content object.
+    """Returns *direct* local roles for passed principal and content object.
     """
     ctype = ContentType.objects.get_for_model(obj)
 
@@ -224,72 +278,9 @@ def get_local_roles(obj, principal):
         return [prr.role for prr in PrincipalRoleRelation.objects.filter(
             group=principal, content_id=obj.id, content_type=ctype)]
 
-def has_role(principal, role, obj=None):
-    """Returns True if the passed principal has passed role. If an object is
-    passed local roles will also taking into account.
-
-    **Parameters:**
-
-    principal
-        The principal (user or group) for which the roles are checked.
-
-    role
-        The role which is checked. Either the role name or the Role instance.
-
-    obj
-        The object for which the role is checked
-    """
-    if isinstance(role, str):
-        role = Role.objects.get(name=role)
-
-    roles = get_roles(principal, obj)
-    return role in roles
-
-def has_local_role(principal, role, obj):
-    """Returns True if the passed principal has the passed role for passed
-    object.
-
-    **Parameters:**
-
-    principal
-        The principal (user or group) for which the roles are checked.
-
-    role
-        The role which is checked. Either the role name or the Role instance.
-
-    obj
-        The object for which the role is checked
-    """
-    if isinstance(role, str):
-        role = Role.objects.get(name=role)
-
-    roles = get_local_roles(obj, principal)
-    return role in roles
-
-def has_global_role(principal, role):
-    """Returns True if the passed principal has the global role for passed
-    object.
-    
-    **Parameters:**
-
-    principal
-        The principal (user or group) for which the roles are checked.
-
-    role
-        The role which is checked. Either the role name or the Role instance.
-
-    obj
-        The object for which the role is checked
-    """
-    if isinstance(role, str):
-        role = Role.objects.get(name=role)
-
-    roles = get_roles(principal)
-    return role in roles
-
 # Permissions ################################################################
 
-def check_permission(obj, user, permission, roles=None):
+def check_permission(obj, user, codename, roles=None):
     """Checks whether passed user has passed permission for passed obj.
 
     **Parameters:**
@@ -307,8 +298,8 @@ def check_permission(obj, user, permission, roles=None):
         If given these roles will be assigned to the user temporarily before
         the permissions are checked.
     """
-    if not has_permission(obj, user, permission):
-        raise Unauthorized("User '%s' doesn't have permission '%s' for object '%s' (%s)"
+    if not has_permission(obj, user, codename, roles):
+        raise Unauthorized("User '%s' doesn't have permission '%s' for object '/%s' (%s)."
             % (user, codename, obj.slug, obj.__class__.__name__))
 
 def grant_permission(obj, role, permission):
@@ -384,43 +375,45 @@ def has_permission(obj, user, codename, roles=None):
     codename
         The permission's codename which should be checked.
 
-    user
-        The user for which the permission should be checked.
+    request
+        The current request.
 
     roles
         If given these roles will be assigned to the user temporarily before
         the permissions are checked.
     """
-    if roles is None:
-        roles = []
-
     if user.is_superuser:
         return True
 
-    if user.is_anonymous():
-        user = None
-    else:
+    if roles is None:
+        roles = []
+
+    if not user.is_anonymous():
         roles.extend(get_roles(user, obj))
 
-    ct = ContentType.objects.get_for_model(obj)
+    ctype = ContentType.objects.get_for_model(obj)
 
+    result = False
     while obj is not None:
         p = ObjectPermission.objects.filter(
-            content_type=ct, content_id=obj.id, role__in=roles, permission__codename = codename)
+            content_type=ctype, content_id=obj.id, role__in=roles, permission__codename = codename).values("id")
 
-        if p.count() > 0:
-            return True
+        if len(p) > 0:
+            result = True
+            break
 
         if is_inherited(obj, codename) == False:
-            return False
+            result = False
+            break
 
         try:
             obj = obj.get_parent_for_permissions()
-            ct = ContentType.objects.get_for_model(obj)
+            ctype = ContentType.objects.get_for_model(obj)
         except AttributeError:
-            return False
+            result = False
+            break
 
-    return False
+    return result
 
 # Inheritance ################################################################
 
@@ -446,7 +439,7 @@ def add_inheritance_block(obj, permission):
         ObjectPermissionInheritanceBlock.objects.get(content_type = ct, content_id=obj.id, permission=permission)
     except ObjectPermissionInheritanceBlock.DoesNotExist:
         try:
-            result = ObjectPermissionInheritanceBlock.objects.create(content=obj, permission=permission)
+            ObjectPermissionInheritanceBlock.objects.create(content=obj, permission=permission)
         except IntegrityError:
             return False
     return True
@@ -488,41 +481,71 @@ def is_inherited(obj, codename):
         The content object for which the permission should be checked.
 
     codename
-        The permission which should be checked. Must be the codename of the 
+        The permission which should be checked. Must be the codename of the
         permission.
     """
-    ct = ContentType.objects.get_for_model(obj)
+    ctype = ContentType.objects.get_for_model(obj)
     try:
         ObjectPermissionInheritanceBlock.objects.get(
-            content_type=ct, content_id=obj.id, permission__codename = codename)
+            content_type=ctype, content_id=obj.id, permission__codename = codename)
     except ObjectDoesNotExist:
         return True
     else:
         return False
 
-def get_group(id):
-    """Returns the group with passed id or None.
+def get_group(id_or_name):
+    """Returns the group with passed id or name. If it not exists it returns
+    None.
     """
     try:
-        return Group.objects.get(pk=id)
-    except Group.DoesNotExist:
-        return None
+        return Group.objects.get(pk=id_or_name)
+    except (Group.DoesNotExist, ValueError):
+        try:
+            return Group.objects.get(name=id_or_name)
+        except Group.DoesNotExist:
+            return None
 
-def get_role(id):
-    """Returns the role with passed id or None.
-    """
-    try:
-        return Role.objects.get(pk=id)
-    except Role.DoesNotExist:
-        return None
+def get_role(id_or_name):
+    """Returns the role with passed id or name. If it not exists it returns
+    None.
 
-def get_user(id):
-    """Returns the user with passed id or None.
+    **Parameters:**
+
+    id_or_name
+        The id or the name of the role which should be returned.
     """
     try:
-        return User.objects.get(pk=id)
-    except User.DoesNotExist:
-        return None
+        return Role.objects.get(pk=id_or_name)
+    except (Role.DoesNotExist, ValueError):
+        try:
+            return Role.objects.get(name=id_or_name)
+        except Role.DoesNotExist:
+            return None
+
+def get_user(id_or_username):
+    """Returns the user with passed id or username. If it not exists it returns
+    None.
+
+    **Parameters:**
+
+    id_or_username
+        The id or the username of the user which should be returned.
+    """
+    try:
+        return User.objects.get(pk=id_or_username)
+    except (User.DoesNotExist, ValueError):
+        try:
+            return User.objects.get(username=id_or_username)
+        except User.DoesNotExist:
+            return None
+
+def has_group(user, group):
+    """Returns True if passed user has passed group.
+    """
+    if isinstance(group, str):
+        group = Group.objects.get(name=group)
+
+    return group in user.groups.all()
 
 def reset(obj):
     """Resets all permissions and inheritance blocks of passed object.
@@ -590,7 +613,7 @@ def register_role(name):
 
     name
         The unique role name.
-    """    
+    """
     role, created = Role.objects.get_or_create(name=name)
     if created:
         return role
@@ -647,4 +670,4 @@ def unregister_group(name):
         return False
 
     group.delete()
-    return True
\ No newline at end of file
+    return True
